package binaryRead;

import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.IOException;

/**
 * This class demonstrates how to use BufferedInputStream to efficiently read
 * raw bytes from a binary file.
 *
 * Key Concepts:
 * - BufferedInputStream adds a memory buffer between the program and the file system.
 *   This reduces the number of direct disk reads, which can improve performance.
 * - You still read one byte at a time using read(), but the buffer fetches a larger
 *   chunk of data behind the scenes.
 *
 * Note:
 * - This still reads unstructured byte data (like FileInputStream).
 * - The binary data will look like raw numbers, not human-readable text.
 * - If the file was written using DataOutputStream, each value may take multiple bytes.
 */
public class BufferedInputStreamExample {

    public static void main(String[] args) {
        FileInputStream fis = null;
        BufferedInputStream bis = null;

        try {
            // Open the binary file for reading
            fis = new FileInputStream("data.bin");

            // Wrap it in a BufferedInputStream for more efficient reading
            bis = new BufferedInputStream(fis);

            int byteValue;
            int byteCount = 0;

            // Read one byte at a time, until end of file is reached (-1)
            while ((byteValue = bis.read()) != -1) {
                System.out.printf("Byte %2d: %3d\n", byteCount, byteValue);
                byteCount++;
            }

            System.out.println("Finished reading " + byteCount + " bytes (buffered).");

        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            // Always close the stream (this also closes the underlying FileInputStream)
            try {
                if (bis != null) {
                    bis.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}

