package binaryRead;

import java.io.DataInputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * This class demonstrates how to use DataInputStream to read primitive data types
 * from a binary file that was previously written using DataOutputStream.
 *
 * Stream Chain:
 * FileInputStream → BufferedInputStream → DataInputStream
 *
 * What It Does:
 * - Reads binary data and reconstructs the original Java primitive types.
 * - MUST read in the same order and type as they were written.
 * - Prints the values to the console in readable form.
 *
 * Important:
 * - If you open the binary file (e.g., "data.bin") in a text editor, you will NOT see
 *   readable characters. The bytes represent raw values (not printable text).
 */
public class DataInputStreamExample {

    public static void main(String[] args) {
        try {
            // Open the binary file "data.bin" for reading
            FileInputStream fis = new FileInputStream("data.bin");

            // Wrap it in a buffered stream for performance
            BufferedInputStream bis = new BufferedInputStream(fis);

            // Wrap in DataInputStream to read Java primitives
            DataInputStream dis = new DataInputStream(bis);

            // Read in the same order/types as written in DataOutputStream
            int number = dis.readInt();           // 4 bytes
            double pi = dis.readDouble();         // 8 bytes
            boolean flag = dis.readBoolean();     // 1 byte
            String message = dis.readUTF();       // 2 bytes for length + UTF-8 chars

            // Output the read values to verify correctness
            System.out.println("Read int:      " + number);
            System.out.println("Read double:   " + pi);
            System.out.println("Read boolean:  " + flag);
            System.out.println("Read UTF text: " + message);

            // Close the stream (automatically closes all wrapped streams)
            dis.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
