package binaryWrite;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * This class demonstrates the use of BufferedOutputStream for writing bytes to a file.
 *
 * BufferedOutputStream adds a memory buffer between your program and the file.
 * When you write bytes, they go into the buffer first. The buffer is only written to
 * the file when it's full or when you explicitly call flush() or close().
 *
 * This can significantly improve performance by reducing the number of slow disk operations.
 */
public class BufferedOutputDemo {
    public static void main(String[] args) {
        String filename = "buffered_output.bin"; // Output file

        try (
            // Step 1: Create a raw byte output stream to the file
            FileOutputStream fos = new FileOutputStream(filename);

            // Step 2: Wrap it in a BufferedOutputStream to add buffering
            BufferedOutputStream bos = new BufferedOutputStream(fos)
        ) {
            /**
             * Write a few bytes (values from 0 to 9) to the buffered stream.
             * These bytes are not written directly to the file yet —
             * they are stored in the buffer.
             */
            for (int i = 0; i < 10; i++) {
                bos.write(i); // Writes a single byte to the buffer
            }

            /**
             * At this point, the file might still be empty or partially written
             * depending on the size of the buffer and the operating system's behavior.
             *
             * To ensure all buffered data is written to the actual file, we call flush().
             * This forces any buffered bytes to be sent to the underlying FileOutputStream.
             */
            bos.flush();

            System.out.println("Data successfully written to " + filename);

        } catch (IOException e) {
            System.err.println("Error writing to file: " + e.getMessage());
        }

        /**
         * Note: Even without an explicit call to flush(), closing the stream
         * will automatically flush any remaining data in the buffer.
         */
    }
}
